/*==============================================================*
 * slips.c	- main module for slips.
 *
 * Coypright (C) 1994, 1995 by O.Kirch, okir@monad.swb.de
 * For copying conditions, see the file COPYING.
 *==============================================================*/

#include <stdlib.h>
#include <tcl/tcl.h>
#include "slipsh.h"

/*==============================================================*
 * Tcl_AppInit	initialize tcl application
 * This function is called from the libtcl main routine.
 *==============================================================*/
int
Tcl_AppInit(Tcl_Interp *interp)
{
    Tcl_CreateCommand(interp, "tty", tty_all, (ClientData) 0,
    		      (Tcl_CmdDeleteProc*)NULL);
    Tcl_CreateCommand(interp, "net", net_all, (ClientData) 0,
    		      (Tcl_CmdDeleteProc*)NULL);
    Tcl_CreateCommand(interp, "sys", sys_all, (ClientData) 0,
    		      (Tcl_CmdDeleteProc*)NULL);
    return TCL_OK;
}

/*==============================================================*
 * tty_all	handle TCL tty commands
 * Usage:
 *	tty <foo> <options>
 *==============================================================*/
int
tty_all(ClientData cd, Tcl_Interp *interp, int argc, char *argv[])
{
  char	*minor, c;

  Tcl_ResetResult(interp);

  if (argc < 2) {
	sprintf(interp->result, "%s: desire minor tcl command", argv[0]);
	return TCL_ERROR;
  }
  minor = argv[1]; c = *minor;
  if (c == 'c' && !strcmp(minor, "chat")) {
	return tty_chat(cd, interp, argc-1, argv+1);
  }
  if (c == 'c' && !strcmp(minor, "close")) {
	return tty_close(cd, interp, argc-1, argv+1);
  }
  if (c == 'e' && !strcmp(minor, "expect")) {
	return tty_expect(cd, interp, argc-1, argv+1);
  }
  if (c == 'f' && !strcmp(minor, "flush")) {
	return tty_flush(cd, interp, argc-1, argv+1);
  }
  if (c == 'h' && !strcmp(minor, "hangup")) {
	return tty_hangup(cd, interp, argc-1, argv+1);
  }
  if (c == 'l' && !strcmp(minor, "lock")) {
	return tty_lock(cd, interp, argc-1, argv+1);
  }
  if (c == 'o' && !strcmp(minor, "open")) {
	return tty_open(cd, interp, argc-1, argv+1);
  }
  if (c == 'r' && !strcmp(minor, "recv")) {
	return tty_recv(cd, interp, argc-1, argv+1);
  }
  if (c == 's' && !strcmp(minor, "send")) {
	return tty_send(cd, interp, argc-1, argv+1);
  }
  if (c == 'u' && !strcmp(minor, "unlock")) {
	return tty_unlock(cd, interp, argc-1, argv+1);
  }
  sprintf(interp->result, "%s: unknown minor tcl command", argv[0]);
  return TCL_ERROR;
}

/*==============================================================*
 * net_all	handle TCL net commands
 * Usage:
 *	net <foo> <options>
 *==============================================================*/
int
net_all(ClientData cd, Tcl_Interp *interp, int argc, char *argv[])
{
  char	*minor, c;

  Tcl_ResetResult(interp);

  if (argc < 2) {
	sprintf(interp->result, "%s: desire minor tcl command", argv[0]);
	return TCL_ERROR;
  }
  minor = argv[1]; c = *minor;
  if (c == 'i' && !strcmp(minor, "iflist")) {
	return net_iflist(cd, interp, argc-1, argv+1);
  }
  if (c == 'i' && !strcmp(minor, "ifstats")) {
	return net_ifstats(cd, interp, argc-1, argv+1);
  }
  if (c == 's' && !strcmp(minor, "slip")) {
	return net_slip(cd, interp, argc-1, argv+1);
  }
  sprintf(interp->result, "%s: unknown minor tcl command", argv[0]);
  return TCL_ERROR;
}

/*==============================================================*
 * sys_all	handle TCL sys commands
 * Usage:
 *	sys <foo> <options>
 *==============================================================*/
int
sys_all(ClientData cd, Tcl_Interp *interp, int argc, char *argv[])
{
  char	*minor, c;

  Tcl_ResetResult(interp);

  if (argc < 2) {
	sprintf(interp->result, "%s: desire minor tcl command", argv[0]);
	return TCL_ERROR;
  }
  minor = argv[1]; c = *minor;
  if (c == 'f' && !strcmp(minor, "fork")) {
	return sys_fork(cd, interp, argc-1, argv+1);
  }
  if (c == 'k' && !strcmp(minor, "kill")) {
	return sys_kill(cd, interp, argc-1, argv+1);
  }
  if (c == 'p' && !strcmp(minor, "pause")) {
	return sys_pause(cd, interp, argc-1, argv+1);
  }
  if (c == 's' && !strcmp(minor, "signal")) {
	return sys_signal(cd, interp, argc-1, argv+1);
  }
  if (c == 's' && !strcmp(minor, "sleep")) {
	return sys_sleep(cd, interp, argc-1, argv+1);
  }
  if (c == 't' && !strcmp(minor, "time")) {
	return sys_time(cd, interp, argc-1, argv+1);
  }
  sprintf(interp->result, "%s: unknown minor tcl command", argv[0]);
  return TCL_ERROR;
}


/*==============================================================*
 * tcl_getopt	simple getopt functionality for TCL commands.
 *==============================================================*/
int
tcl_getopt(Tcl_Interp *interp, struct option *option_table,
	   int argc, char *argv[], int *optind)
{
  struct option	*tbl;
  opt_multi	*ap;
  char		*sp;
  int		i = *optind, type, mult;
  void		*optpt;
  char		**strp;
  int		*intp;

  while(i < argc && argv[i][0] == '-') {
  	sp = argv[i]+1;
  	for (tbl = option_table; tbl->name != NULL; tbl++) {
  		if (*sp != tbl->name[0] || strcmp(sp, tbl->name)) {
  			continue;
  		}
  		type = (tbl->type & T_SCALAR);
  		mult = (tbl->type & T_MULTI);
  		if ((type == T_INT || type == T_STRING) && i == argc-1) {
  			sprintf(interp->result,
  				 "%s: option -%s need an argument",
  				  argv[0], sp);
  			*optind = i;
  			return TCL_ERROR;
  		}
  		if (tbl->type & T_MULTI) {
  			ap = (opt_multi*)tbl->argp;
  			if (ap->count >= ap->max) {
  				sprintf(interp->result,
  					"%s: too many occurrences of option -%s",
  					argv[0], sp);
  				*optind = i;
  				return TCL_ERROR;
  			}
  			if (type == T_INT) {
  				optpt = ((char**)(ap->array))+(ap->count++);
  			} else if (type == T_STRING) {
  				optpt = ((int*)(ap->array))+(ap->count++);
  			} else {
  				sprintf(interp->result,
  					"%s: internal error: "
  					"multi option only supported for INT "
  					"and STRING arguments.",
  					argv[0]);
  				*optind = i;
  				return TCL_ERROR;
  			}
  		} else {
  			optpt = tbl->argp;
  		}
  		switch(type) {
  		case T_TRUE:
  			if ((intp = (int*)optpt) != NULL) {
  				*intp = 1;
  			}
  			break;
  		case T_FALSE:
  			if ((intp = (int*)optpt) != NULL) {
  				*intp = 0;
  			}
  			break;
  		case T_INT:
  			if ((intp = (int*)optpt) != NULL) {
  				int	base = 10;
  				sp = argv[++i];
  				if (*sp == '0') {
  					if (*++sp == 'x') {
  						base = 16;
  						sp++;
  					} else {
  						base = 8;
  					}
  				}
  				*intp = strtol(sp, NULL, base);
  			} else {
  				i++;
  			}
  			break;
  		case T_STRING:
  			if ((strp = (char**)optpt) != NULL) {
  				*strp = argv[++i];
  			} else {
  				i++;
  			}
  			break;
  		case T_VOID:
  			break;
  		}
  		break;
  	}
  	if (tbl->name == NULL) {
  		/* an option of -- terminates the argument list */
  		if (!strcmp(sp, "-")) {
  			i++;
  		}
  		return TCL_OK;
  	}
  	i++;
  }
  *optind = i;
  return TCL_OK;
}

